/*

    sftpd - meant to be a small and flexible ftp server implementation
    Copyright (C) 2009  Jan Murawski  der@oelgoetze.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

    For more information about the GNU Public license see:
    http://www.gnu.org/copyleft/gpl.html

*/

#include "rBufferHelper.h"


//
// let's initialize our object
//
rBufferHelper::rBufferHelper(unsigned int space)
{
    totalBufferSpace = space;
    buffer = new char[space];
    beginsAt = 0;
    filledWith = 0;
}

rBufferHelper::~rBufferHelper() 
{ 
    delete buffer; 
}


//
// return position from wich buffer can be read
//
char* rBufferHelper::getUsedBuffer() 
{ 
    return (buffer + beginsAt); 
}


//
// return position from wich buffer can be written
//
char* rBufferHelper::getEmptyBuffer()
{
    // does the buffer content raeach into the first
    // half of the ring?
    if (beginsAt + filledWith > totalBufferSpace)
    {
        // yes. substract buffer size from position 
        return buffer + ((beginsAt + filledWith) - totalBufferSpace);
    }
    else
        return buffer + (beginsAt + filledWith);    
}


//
// return free buffer space
//
unsigned int rBufferHelper::freeSpace() 
{ 
    return (totalBufferSpace - filledWith); 
}


//
// return used buffer space
//
unsigned int rBufferHelper::usedSpace()
{
    return filledWith; 
}


//
// return max bytes wirtable till buffer is full
// or ring border is reached
//
unsigned int rBufferHelper::maxWritable()
{
    // does the buffer content raeach into the first
    // half of the ring?
    if (beginsAt + filledWith > totalBufferSpace)
    {
        // yes. space in the first half is from 0 to start marker
        // so substract buffer reaching in first half from start marker
        return beginsAt - ((beginsAt + filledWith) - totalBufferSpace);
    }
    else
        // space left till ring border
        return totalBufferSpace - (beginsAt + filledWith);
}


//
// return max bytes readable till buffer border
//
unsigned int rBufferHelper::maxReadable()
{
    // does the buffer content raeach into the first
    // half of the ring?
    if (beginsAt + filledWith > totalBufferSpace)
        // bytes till ring border
        return totalBufferSpace - (beginsAt + filledWith);
    else
        return filledWith;
}


//
// user added bytes to buffer
//
bool rBufferHelper::added(unsigned int count)
{
    filledWith += count;
    // do we have enough buffer space for given count?
    // else correct value and return false
    if (filledWith > totalBufferSpace)
    {
        filledWith = totalBufferSpace;
        return false;
    }
    else
        return true;
}


//
// user removed bytes from buffer
//
bool rBufferHelper::removed(unsigned int count)
{
    // are that much bytes in buffer?
    // if no correct value and return false
    if (filledWith < count)
    {
        filledWith = 0;
        beginsAt = 0;
        return false;
    }
    else
    {
        filledWith -= count;                    // adjust fillstate
        beginsAt += count;                      // ad just begining of used space
        // if pointer is at the end of the ring
        // place it at the beginning of the other ring
        if (beginsAt >= totalBufferSpace)
            beginsAt -= totalBufferSpace;
        return true;
    }
}


//
// reset buffer values
//
void rBufferHelper::clear()
{
    filledWith = 0;
    beginsAt = 0;
}
