/*

    sftpd - meant to be a small and flexible ftp server implementation
    Copyright (C) 2009  Jan Murawski  der@oelgoetze.de

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

    For more information about the GNU Public license see:
    http://www.gnu.org/copyleft/gpl.html

*/

#include "serverThreads.h"

#ifndef _SEVER_OBJ_
#define _SEVER_OBJ_

#include <windows.h>
#include <winsock.h>
#pragma comment(lib, "Ws2_32.lib") 

#include <deque>
#include <string>
using namespace std;

#include "socketHelper.h"
#include "directoryObject.h"
#include "convertUtil.h"


//
// a user account
//
struct ftpUser {
    string user;            // login
    string password;        // password
    string folder;          // users root folder
    bool folderWritable;    // write access?
};


//
// struct identifying a data connection granted by a controle thread
//
struct dataConnHandle {
    string remoteIp;            // remote IP allowed to connect. ip is the only identifiying attribute. 
                                // who the hell designed that ***** protokoll ??
    SOCKET *sck;                // assigned socket. should be NULL if none.
    unsigned int initializedAt; // time, at which the request was initiated
    bool timedOut;              // we have waited long enough now...
};


//
// one server instance
//
class ServerObj
{
private:
    CRITICAL_SECTION csl;                           // mangaed access to object vars
    
    deque<ftpUser*> users;                          // list of all registered accounts
    unsigned short controlePort, dataPort;          // the listen ports
    
    bool listenning;                                // running?
    bool getFinished;                               // used to notice threads to close
    
    deque<HANDLE> threadList;                       // list of all registered threads
    void maintainThreadList();

    deque<dataConnHandle*> dchList;                 // list of allowed (desired) data connections


public:
    ServerObj();
    ~ServerObj();

    // designed for extern use --------------------

    void addUser(char *user, char *password,        // add a new user account to our list
            char *folder, bool folderWritable);
    bool setPorts(int controlePort, int dataPort);  // change the port values (needs restart)
    bool start();                                   // start the server
    void stop();                                    // wait for all threads to stop


    // designed for intern use --------------------
    // needs to be public cause of c-style threads
    // and thus massive pointer usage

    bool userExists(string user);                   // test, if given user name is in our list
    bool loginValid(string user, string password);  // test, if given username and password do 
                                                    // match an entry in our list
    int getDataPort();                              // returns data port
    bool getLoginSettings(string user, 
                          string password, 
                          string &directory, 
                          string &rw);

    void regThread(HANDLE hThread);                 // add a new thread to our list, so that we know what is running
    bool getGetFinished();                          // should the server be stopped?


    dataConnHandle* createDataConnHandle(string ip);    // register a new data connection request and return handle
    void            cleanUpDataConnHandle               // delete all memory allocated for handle
                        (dataConnHandle *dch);
    dataConnHandle* deleteDataConnHandleEntry           // delete last data connection request entry matching given ip
                        (string ip);                    // returns entry
    dataConnHandle* deleteDataConnHandleEntry           // delete last data connection request entry matching given handle
                        (dataConnHandle *dch);          // returns entry
    dataConnHandle* findDataConnHandle(string ip);      // returns first (!) handle, if request for given ip is existant
    bool registerSocketWithDCH                          // assign socket to handel. returns false if dch is no more existant or
            (dataConnHandle *dch, SOCKET *sck);         // socket already assigned
    bool isSocketRegistered(dataConnHandle *dch);
    bool resetDataConnHandleTimer(dataConnHandle *dch); // reset timer for given date connection
    void maintainDataConnHandleList();                  // check if any incomming connection is pending for to long now...
    bool isDataConnHandleTimedOut(dataConnHandle *dch); // return, if connection request hast timed out
    unsigned int getDataConnHandleCount();              // returns number of all (waiting, timed out, registered) items in list
};


#endif _SEVER_OBJ_
